/*
    This file is part of l_mon.
    Copyright (C) 2016-2022  Laurent Poirrier

    l_mon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with l_mon.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <string.h>
#include "hutils.h"
#include "log.h"
#include "files.h"

#include "gen_index.html.h"
#include "gen_favicon.ico.h"
#include "gen_icon.png.h"

static const char file_index_html[] = GEN_INDEX_HTML;
static const char file_favicon_ico[] = GEN_FAVICON_ICO;
static const char file_icon_png[] = GEN_ICON_PNG;

const char *file_data(const char **name, const char **type, size_t *size,
	const char *url)
{
	if (strends(url, "/")) {
		if (name)
			*name = "index.html";
		if (type)
			*type = "text/html; charset=UTF-8";
		if (size)
			*size = sizeof(file_index_html) - 1;
		return(file_index_html);
	}

	if (strends(url, "/favicon.ico")) {
		if (name)
			*name = "favicon.ico";
		if (type)
			*type = "image/x-icon";
		if (size)
			*size = sizeof(file_favicon_ico) - 1;
		return(file_favicon_ico);
	}

	if (strends(url, "/icon.png")) {
		if (name)
			*name = "icon.png";
		if (type)
			*type = "image/png";
		if (size)
			*size = sizeof(file_icon_png) - 1;
		return(file_icon_png);
	}

	//LOG("request {%s}\n", req);
	
	if (name)
		*name = "unknown";
	if (type)
		*type = "text/html";
	if (size)
		*size = 0;
	
	return(NULL);
}

