#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <inttypes.h>
#include <errno.h>
#include <string.h>

int main(int argc, char **argv)
{
	if (argc < 4) {
		printf("Usage: c_build {width} {height} {depth}\n");
		return 0;
	}

	int w = strtol(argv[1], NULL, 0);
	int h = strtol(argv[2], NULL, 0);
	int depth = strtol(argv[3], NULL, 0);

	if (depth != 255) {
		printf("Depth must be 255\n");
		return 1;
	}

	uint8_t *d = malloc(w * h);

	if (d == NULL) {
		printf("malloc(%d x %d) failed\n", w, h);
		return 1;
	}

	size_t r = fread(d, w * h, 1, stdin);
       
	if (r != 1) {
		printf("fread(%d): %zd: %s\n", w * h, r, strerror(errno));
		return 1;
	}

	int cw = w / 16;
	int ch = h / 8;

	for (int c = 0; c < 128; c++) {
		uint64_t v = 0;
		uint64_t bit = 1;

		int cx = (c % 16) * cw;
		int cy = (c / 16) * ch;

		for (int i = 0; i < ch; i++) {
			for (int j = 0; j < cw; j++) {
				int on = d[(cy + i) * w + cx + j] >= 128;

				if (on)
					v |= bit;

				bit <<= 1;
				if (bit == 0) {
					printf("0x%" PRIx64 ",", v);
					v = 0;
					bit = 1;
				}
			}
		}

		if ((c % 4) == 3)
			printf("\n");
	}
	
	return 0;
}

