/*
    This file is part of l_mon.
    Copyright (C) 2016-2022  Laurent Poirrier

    l_mon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with l_mon.  If not, see <http://www.gnu.org/licenses/>.
*/
#define _POSIX_C_SOURCE	201605L
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <stddef.h>
#include "hutils.h"

// ***********************************************************************
// 
// ***********************************************************************
/*
int strstarts(const char *str, const char *ref)
{
	size_t len;
	
	len = strlen(ref);
	
	return(strncmp(str, ref, len) == 0);
}

int strinline(const char *str, const char *ref)
{
	const char *ptr;
	const char *eol;
	ptrdiff_t offs, end;
	
	ptr = strstr(str, ref);
	
	if (ptr == NULL)
		return(0);
	
	eol = strstr(str, "\r\n");
	
	if (eol == NULL)
		return(1);
	
	offs = ptr - str;
	end = eol - str;
	
	return(offs < end);
}
*/
int strends(const char *str, const char *ref)
{
	size_t slen, rlen;
	
	slen = strlen(str);
	rlen = strlen(ref);
	
	if (slen < rlen)
		return(0);
	
	return(memcmp(str + slen - rlen, ref, rlen) == 0);
}

// ***********************************************************************
// 
// ***********************************************************************
size_t hget_url(char *dst, size_t dsize, const char *req)
{
	const char *url, *end, *eol;
	size_t ulen, llen;
	
	if (memcmp(req, "GET ", 4) != 0)
		return(0);
	
	url = req + 4;
	
	end = strstr(url, " HTTP/");
	eol = strstr(url, "\r\n");
	
	if ((end == NULL) || (eol == NULL))
		return(0);
	
	ulen = (size_t)(end - url);
	llen = (size_t)(eol - url);
	
	if (ulen > llen)
		return(0);
		
	if (ulen < dsize) {
		memcpy(dst, url, ulen);
		dst[ulen] = 0;
	} else if (dsize > 0) {
		memcpy(dst, url, dsize);
		dst[dsize - 1] = 0;
	}
	
	return(ulen);
}

// ***********************************************************************
// 
// ***********************************************************************
static const char *wday[7] =
	{ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static const char *mon[12] =
	{ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
	"Aug", "Sep", "Oct", "Nov", "Dec" };

// ***********************************************************************
// 
// ***********************************************************************
size_t hheader_ok(char *dst, size_t hsize, time_t rtime,
	const char *ctype, size_t csize, int cflags)
{
	struct tm gmt;
	int r;

	gmtime_r(&rtime, &gmt);
	
	r = snprintf(dst, hsize,
		"HTTP/1.1 200 OK\r\n"
		"Date: %s, %d %s %d %02d:%02d:%02d GMT\r\n"
		"Server: montools/1.0\r\n"
		"%sContent-Length: %zd\r\n"
		"Connection: close\r\n"
		"Content-Type: %s\r\n"
		"%s"
		"\r\n",
		wday[gmt.tm_wday], gmt.tm_mday, mon[gmt.tm_mon],
		gmt.tm_year + 1900, gmt.tm_hour, gmt.tm_min, gmt.tm_sec,
		(cflags & HHEADER_NOSIZE) ? "Ignore-The-" : "",
		csize, ctype,
		(cflags & HHEADER_NOCACHE) ?
			"Cache-control: no-cache\r\n"
			"Cache-control: no-store\r\n"
			"Pragma: no-cache\r\n"
			"Expires: 0\r\n"
			: ""
		);

	return((size_t)r);
}


// ***********************************************************************
// 
// ***********************************************************************
size_t hheader_multi_start(char *dst, size_t hsize, time_t rtime,
	int cflags)
{
	return(hheader_ok(dst, hsize, rtime,
		"multipart/x-mixed-replace; boundary=\"montools-bnd\"",
		0, cflags | HHEADER_NOSIZE));
}

size_t hheader_multi_part(char *dst, size_t hsize, time_t rtime,
	const char *ctype, size_t csize, int cflags)
{
	int r;

	(void)rtime;
	
	r = snprintf(dst, hsize,
		"--montools-bnd"
		"%sContent-Length: %zd\r\n"
		"Content-Type: %s\r\n"
		"\r\n",
		(cflags & HHEADER_NOSIZE) ? "Ignore-The-" : "",
		csize, ctype);

	return((size_t)r);
}


// ***********************************************************************
// 
// ***********************************************************************
static const char *move0 =
"<!doctype html><html><head>\n"
"<title>301 Moved Permanently</title>\n"
"</head><body>\n"
"<h1>Moved Permanently</h1>\n"
"<p>The document has moved <a href=\"";
static const char *move1 =
"/\">here</a>.</p>\n"
"</body></html>\n";

size_t hheader_move(char *dst, size_t hsize, time_t rtime,
	const char *url)
{
	struct tm gmt;
	size_t len;
	int r;

	gmtime_r(&rtime, &gmt);
	
	len = strlen(move0) + strlen(url) + strlen(move1);
	
	r = snprintf(dst, hsize,
		"HTTP/1.1 301 Moved Permanently\r\n"
		"Date: %s, %d %s %d %02d:%02d:%02d GMT\r\n"
		"Server: montools/1.0\r\n"
		"Location: %s/\r\n"
		"Content-Length: %zd\r\n"
		"Connection: close\r\n"
		"Content-Type: text/html; charset=UTF-8\r\n"
		"\r\n%s%s%s",
		wday[gmt.tm_wday], gmt.tm_mday, mon[gmt.tm_mon],
		gmt.tm_year + 1900, gmt.tm_hour, gmt.tm_min, gmt.tm_sec,
		url, len, move0, url, move1);
	
	return((size_t)r);
}

/*
*/