/*
    This file is part of l_mon.
    Copyright (C) 2016-2022  Laurent Poirrier

    l_mon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with l_mon.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include "log.h"
#include "refbuf.h"

void pool_init(pool *p)
{
	p->ptr.seq = -1;
	p->ptr.refs = 0;
	p->ptr.data = NULL;
	p->ptr.size = 0;
	p->ptr.prev = &p->ptr;
	p->ptr.next = &p->ptr;
	
	p->seq = 0;
}

void pool_clear(pool *p)
{
	ASSERT(p->ptr.prev == &p->ptr);
	ASSERT(p->ptr.next == &p->ptr);
}

refbuf *refbuf_create(pool *p, size_t n)
{
	refbuf *b;
	
	b = (refbuf *)malloc(sizeof(refbuf));
	
	if (!b) {
		log_time("refbuf_create(): malloc(): %s\n", strerror(errno));
		return(NULL);
	}
	
	b->seq = p->seq;
	b->refs = 1;

	if (n) {
		b->data = malloc(n);
		if (!b->data) {
			log_time("refbuf_create(): malloc(%zd): %s\n",
				n, strerror(errno));
			free(b);
			return(NULL);
		}
		b->size = n;
	} else {
		b->data = NULL;
		b->size = 0;
	}

	b->prev = p->ptr.prev;
	b->next = &p->ptr;
	
	p->ptr.prev->next = b;
	p->ptr.prev = b;
	
	p->seq++;
	
	return(b);
}


void refbuf_ref(refbuf *b, int count)
{
	b->refs += count;
	
	ASSERT(b->refs >= 0);
	
	if (b->refs > 0)
		return;
	
	free(b->data);
	
	b->prev->next = b->next;
	b->next->prev = b->prev;
	
	free(b);
}


