/*
    This file is part of l_mon.
    Copyright (C) 2016-2022  Laurent Poirrier

    l_mon is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with l_mon.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef SERVER_H
#define SERVER_H
#include <stdlib.h>
#include <time.h>
#include "config.h"
#include "network.h"
#include "video_dev.h"
#include "refbuf.h"
#include "mask.h"
#include "roll.h"

#define REQ_SIZE		1024
#define HEADER_SIZE		1024

#define CLIENT_KILL		1
#define CLIENT_IN		2
#define CLIENT_OUT		4
#define CLIENT_LAST_MSG		8
#define CLIENT_E_FRAME		16
#define CLIENT_E_VIDEO		32

typedef struct client_t {
	int fd;
	net_addr addr;
	
	int flags;
	
	char buff[REQ_SIZE];
	size_t pos;
	
	char hbuf[HEADER_SIZE];
	size_t hsize, hpos;
	
	refbuf *fbuf;
	size_t fpos;
	int fseq;
	
	struct client_t *prev, *next;
} client;

typedef struct server_t {
	config *conf;
	
	time_t curtime;
	
	video_device fdev;
	mask_data fmask;
	pool fpool;
	refbuf *fcur;
	int fready;
	time_t fatime;
	int iw, ih;
	
	client d;
	int nd;
	pool dpool;
	
	roll pixroll;

	time_t logtime;
	int recv_frames, sent_frames;
	size_t sent_bytes, sent_bytes_0;
	
	time_t switch_time;
	time_t wakeup_delay;
	int switch_frame;
	
	size_t yuv_buff_size, bkg_buff_size;
	size_t jpeg_buff_size;
	unsigned char *yuv_buff;
	unsigned char *bkg_buff;
	unsigned char *frg_buff;
	void *jpeg_buff;
	
	char url[REQ_SIZE];
} server;


int  server_loop(config *c);

#endif

